package app.mar.activities;

import android.content.Intent;
import android.graphics.Typeface;
import android.os.Bundle;
import android.support.design.widget.FloatingActionButton;
import android.support.design.widget.Snackbar;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.Toolbar;
import android.view.View;
import android.widget.TextView;
import android.widget.Toast;

import java.util.Timer;
import java.util.TimerTask;

import app.mar.game.Bomb;
import app.mar.game.Game;
import app.mar.game.scheduler.SpecialTask;

public class BombActivity extends AppCompatActivity {

    protected Bomb mBomb;
    protected Game mGame;
    protected int mLength;
    protected String mTyped;
    protected TextView mTvPassword;
    protected TextView mTvCar;
    protected TextView mTvBomb;


    /**
     * Si vrai, c'est que c'est la fin du jeu
     * sinon c'est un essai sans avoir tout
     */
    protected boolean mIsEnd = false;

    public static final int KEY_ERASE=-1;
    public static final int KEY_OK=10;


    private Timer mTimer;
    private Runnable mTimerTick = new Runnable() {
        public void run() {
            if(!mGame.getBomb().hasExplosed())
            {
                mTvCar.setText(mGame.getBomb().getTimeBeforeBoom());
            }else
            {
                mTvCar.setText("BOOM !");
                mGame.getScheduler().clear();
                mGame.getScheduler().append(SpecialTask.FINISHED_FAILED);
                mTimer.cancel();
                finish();
            }
        }
    };

    private void timerCallback()
    {
        this.runOnUiThread(mTimerTick);
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_bomb);
        mGame = Game.game();
        mBomb = mGame.getBomb();
        mLength=mBomb.getPasswordLength();
        mTyped="";
        mTvBomb=findViewById(R.id.tv_bombe);
        mTvPassword= findViewById(R.id.tv_code);
        mTvCar= findViewById(R.id.tv_car);

        mTvBomb.setText(mBomb.getPasswordKnown());

        Typeface face = Typeface.createFromAsset(getAssets(), "fonts/font.ttf");
        mTvPassword.setTypeface(face);
        mTvCar.setTypeface(face);

        mTvPassword.setText(completeTypedString());
        mTimer = new Timer();
        mTimer.schedule(new TimerTask() {
            @Override
            public void run() {
                timerCallback();
            }

        }, 0, 1000);
    }

    protected String completeTypedString()
    {
        String t = "";
        for(int i=0; i<mLength; i++)
            t+=(i<mTyped.length())?mTyped.charAt(i):'-';

        return t;
    }

    protected void updatePassword()
    {
        String str = completeTypedString();
        mTvPassword.setText(str);
    }

    protected void erase()
    {
        if(mTyped.length()<1) return;
        String nouveau = "";
        for(int i=0; i<mTyped.length()-1; i++)
            nouveau+=mTyped.charAt(i);
        mTyped=nouveau;
        updatePassword();
    }

    protected void valid()
    {
        if(mLength == mTyped.length())
        {
            if(mBomb.disarm(mTyped))
            { //code bon
                mGame.getScheduler().append(SpecialTask.FINISHED_SUCESS);
                setResult(0);
                finish();
                Toast.makeText(this, "OK !", Toast.LENGTH_LONG).show();
            }else
            { //code mauvais
                Toast.makeText(this, "Raté !", Toast.LENGTH_LONG).show();
            }
        }else
        Toast.makeText(this, "Mauvaise longeur !", Toast.LENGTH_SHORT).show();
    }

    @Override
    public void onBackPressed()
    {
        setResult(0);
        finish();
    }

    protected void enterKey(char c)
    {
        if(mTyped.length()<mLength)
            mTyped+=c;
        updatePassword();
    }


    private void onButton(int k)
    {
        String cc ="0123456789";
        if(k>=0 && k<=9) enterKey(cc.charAt(k));
        else if(k==KEY_ERASE) erase();
        else if(k==KEY_OK) valid();
    }

    public void onButton0(View v) {onButton(0);}
    public void onButton1(View v) {onButton(1);}
    public void onButton2(View v) {onButton(2);}
    public void onButton3(View v) {onButton(3);}
    public void onButton4(View v) {onButton(4);}
    public void onButton5(View v) {onButton(5);}
    public void onButton6(View v) {onButton(6);}
    public void onButton7(View v) {onButton(7);}
    public void onButton8(View v) {onButton(8);}
    public void onButton9(View v) {onButton(9);}
    public void onButtonEff(View v) {onButton(KEY_ERASE);}
    public void onButtonOk(View v) {onButton(KEY_OK);}
    public static final int REQUEST_CODE = 24;

}


