package app.mar.activities.fragments;

import android.app.Activity;
import android.app.Fragment;
import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.Typeface;
import android.net.Uri;
import android.os.Bundle;
import android.util.Log;
import android.util.TypedValue;
import android.view.KeyEvent;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;
import android.widget.Toast;

import java.util.ArrayList;
import java.util.Timer;

import app.mar.activities.ChallengeDriver;
import app.mar.activities.R;
import app.mar.exceptions.ResourceFileNotFoundException;
import app.mar.exceptions.ResourceNotFoundException;
import app.mar.game.Resource;
import app.mar.game.ResourceManager;
import app.mar.game.challenges.Challenge;
import app.mar.game.Indice;
import app.mar.utils.files.FileManager;
import uk.co.senab.photoview.PhotoViewAttacher;

public abstract class ChallengeFragment extends Fragment {
    public abstract boolean check();
    protected abstract  void onResetUi();



    protected LinearLayout  mUITextIndices;
    protected LinearLayout  mUIGlobalIndice;
    protected Challenge     mChallenge;
    protected ImageView     mImage;
    private PhotoViewAttacher mAttacher;

    protected void onInitChallenge(Challenge c)
    {
        Log.e("--------","Init challenge '"+c.getName()+"'  -> '"+c.getResourceName()+"'");
        if(c!=null) setImageResource(c.getResourceName());
    }

    public void onChallengeFailed()
    {

    }

    public void onChallengePassed()
    {

    }

    public boolean onBackPressed()
    {
        return true;
    }

    protected void setImageResource(String res)
    {
        Resource r = ResourceManager.getInstance().get(res);

        if(r==null)
        {
            throw new ResourceNotFoundException(res);
        }

        if( r.isImage() )
        {
            String name = r.getName();
            String path = r.getFile();

            Bitmap bmp=null;
            try {
                 bmp = FileManager.openImage(getActivity(), path);
                 Log.e("------", "File found : "+path);
            } catch (Exception e) {
                e.printStackTrace();
                throw new ResourceFileNotFoundException(r);
            }

            mImage.setImageBitmap(bmp);
            mAttacher = new PhotoViewAttacher(mImage);
            mAttacher.setMaximumScale(20);
        }
    }

    public void resetUi()
    {
        getDriver().setVisibilityView(R.id.btn_indice, View.VISIBLE);
        if(mUITextIndices!=null)
        {
            mUITextIndices.removeAllViews();
        }
        if(mUIGlobalIndice!=null)
        {
            mUIGlobalIndice.setVisibility(View.GONE);
        }
        onResetUi();
    }

    protected TextView createTextView(Indice ind, int indInd)
    {
        TextView tv = new TextView(getActivity());
        tv.setTextColor(0xff70e030);
        tv.setTextSize(TypedValue.COMPLEX_UNIT_SP, 14);
        tv.setText("  Indice "+(indInd+1)+" : "+ind.getText());
        tv.setTypeface(null, Typeface.BOLD);
        return  tv;
    }

    private void updateIndices()
    {
        ArrayList<Indice> inds = mChallenge.getIndices();
        int indInd = mChallenge.getIndiceIndex();
        if(inds.size()>0) mUIGlobalIndice.setVisibility(View.VISIBLE);
        for(int i=0; i<inds.size(); i++)
        {
            Indice ind = inds.get(i);
            if(!ind.isDiscovered()) return;
            mUITextIndices.addView(createTextView(ind, indInd));

        }
        //si on arrive ici, ca veut dire que tous les indices ont ete découvert
        getDriver().setVisibilityView(R.id.btn_indice, View.INVISIBLE);
    }

    public Indice showIndice()
    {
        Indice ind = mChallenge.nextIndice();
        int indInd = mChallenge.getIndiceIndex();
        if(ind!=null)
        {
            if(getActivity() instanceof ChallengeDriver)
            {

                    if(mUITextIndices!=null) {
                        mUIGlobalIndice.setVisibility(View.VISIBLE);
                        mUITextIndices.addView(createTextView(ind, indInd));
                        ind.discover();
                        return ind;
                    }
            }
        }
        return null;
    }





    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
    }

    public View onCreateView(LayoutInflater inflater, ViewGroup container,int res) {
        // Inflate the layout for this fragment
        View v = inflater.inflate(res, container, false);
        mUITextIndices= (LinearLayout) v.findViewById(R.id.ll_textIndices);
        mUIGlobalIndice = (LinearLayout) v.findViewById(R.id.ll_rootIndices);
        mImage = (ImageView) v.findViewById(R.id.iv_img);
        return v;
    }


    public void initChallenge(Challenge c)
    {
        if(c==null)
        {
            getDriver().setFinished();
        }
        mChallenge=c;
        resetUi();
        if(mChallenge!=null && !mChallenge.hasNextIndice())
        {
            getDriver().setVisibilityView(R.id.btn_indice, View.INVISIBLE);
        }
        onInitChallenge(c);
    }

    public ChallengeDriver getDriver()
    {
        Activity act = getActivity();
        if(act instanceof ChallengeDriver)
            return (ChallengeDriver)act;
        else if(act==null)
            return null;
        throw new RuntimeException("Error Activty hosting this Fragment ("+getClass()+") is not a ChallengeDriver");
    }




    protected void ready()
    {
        getDriver().onReady(getDriver().currentChallenge());
    }

    @Override
    public void onStart() {
        super.onStart();
        ready();
    }


    public void onResume() {
        super.onResume();
        if(mChallenge!=null) updateIndices();
    }


    public boolean onKeyDown(int keyCode, KeyEvent event) {
        return false;
    }

    private OnFragmentInteractionListener mListener;


    public interface OnFragmentInteractionListener {
        void onFragmentInteraction(Uri uri);
    }

    public void onButtonPressed(Uri uri) {
        if (mListener != null) {
            mListener.onFragmentInteraction(uri);
        }
    }

    @Override
    public void onAttach(Context context) {
        super.onAttach(context);
        if (context instanceof OnFragmentInteractionListener) {
            mListener = (OnFragmentInteractionListener) context;
        } else {
            throw new RuntimeException(context.toString()
                    + " must implement OnFragmentInteractionListener");
        }

    }

    @Override
    public void onDetach() {
        super.onDetach();
        mListener = null;
    }

    public Challenge getChallenge() {
        return mChallenge;
    }




    //
    // Fin de Timer géolacalisation
    //


}
