package app.mar.activities.viewers;

import android.media.MediaPlayer;
import android.os.Bundle;
import android.os.Handler;
import android.support.v7.app.AppCompatActivity;
import android.view.View;
import android.widget.Button;
import android.widget.FrameLayout;
import android.widget.ImageButton;
import android.widget.ImageView;
import android.widget.SeekBar;
import android.widget.TextView;
import android.widget.Toast;

import app.mar.activities.R;
import app.mar.exceptions.BadResourceTypeException;
import app.mar.exceptions.ResourceFileNotFoundException;
import app.mar.game.Resource;
import app.mar.ui.MediaView;


public class TestVideoActivity extends AppCompatActivity implements MediaPlayer.OnSeekCompleteListener,
        MediaView.OnStartListener,
        SeekBar.OnSeekBarChangeListener {
    public static final int REQUEST_CODE = 28;
    private Resource mResource;
    private Button mInfo;
    private TextView mTitle;

    private FrameLayout mFrameLayout;
    private ImageButton  mButton;
    private MediaView mMediaView;
    private ImageView mImageView;
    private SeekBar mBar;
    private Handler mHandler = new Handler();
    private TestVideoActivity.AutoSeek mAutoSeek;
    private int mCurrentTime;
    private int mEndTime;
    private boolean mUserSeeks=false;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_test_video);

        mFrameLayout = (FrameLayout) findViewById(R.id.media_viewer);
        mButton = (ImageButton) findViewById(R.id.play);
        mInfo = (Button) findViewById(R.id.info);
        mBar = (SeekBar) findViewById(R.id.seekBar);
        mBar.setOnSeekBarChangeListener(this);

        mTitle = (TextView) findViewById(R.id.title);
        mAutoSeek = new TestVideoActivity.AutoSeek(mHandler);

        mMediaView = new MediaView(this);
        mMediaView.setOnStartListener(this);
        mImageView = new ImageView(this);
        mImageView.setImageResource(R.drawable.audio);

        mFrameLayout.addView(mMediaView);

        if(getIntent().hasExtra("resource"))
        {
            Toast.makeText(this, "OK", Toast.LENGTH_LONG).show();
            mResource = (Resource) getIntent().getSerializableExtra("resource");
            if(mResource==null)
            {
                throw  new RuntimeException("Ressource non valide");
            }

            mTitle.setText(mResource.getTitle());

            String path=mResource.getName();
            if(mResource.isAudio())
            {
                mImageView.setVisibility(View.VISIBLE);
            }
            else if(mResource.isVideo())
            {
                mImageView.setVisibility(View.GONE);
            }
            else
            {
                throw new BadResourceTypeException(mResource, Resource.AUDIO+" ou "+Resource.VIDEO);
            }

            boolean b = mMediaView.play(path);
            if(!b) throw new ResourceFileNotFoundException(mResource);

        }else {
            throw  new RuntimeException("La ressource n'a pas été passé sous la clé 'resource'");
        }
            Toast.makeText(this, "FAIL", Toast.LENGTH_LONG).show();


    }

    public void onFullscreen(View v)
    {
        if(mTitle.getVisibility()==View.VISIBLE)
        {
            mTitle.setVisibility(View.GONE);
            mInfo.setVisibility(View.GONE);
        }else
        {
            mTitle.setVisibility(View.VISIBLE);
            mInfo.setVisibility(View.VISIBLE);
        }
    }

    public void onPlayPause(View v)
    {
        if(mMediaView.isPlaying())
        {
            mAutoSeek.stop();
            mButton.setImageResource(android.R.drawable.ic_media_play);
            mMediaView.pause();
        }else
        {
            mAutoSeek.start();
            mButton.setImageResource(android.R.drawable.ic_media_pause);
            mMediaView.start();
        }
    }

    public void onInfos(View v)
    {

    }

    @Override
    public void onSeekComplete(MediaPlayer mediaPlayer) {
        mAutoSeek.start();
        mButton.setImageResource(android.R.drawable.ic_media_pause);
    }



    public void onUpdateSeek()
    {
        if(mMediaView.isPlaying())
        {
            mCurrentTime=mMediaView.getTime()/1000;
            mBar.setProgress(mCurrentTime);
        }
    }

    @Override
    public void onProgressChanged(SeekBar seekBar, int i, boolean b) {
        mUserSeeks=b;
    }

    @Override
    public void onStartTrackingTouch(SeekBar seekBar) {

    }


    @Override
    public void onStopTrackingTouch(SeekBar seekBar) {
        if(mUserSeeks)
        {
            mCurrentTime=seekBar.getProgress();
            mMediaView.seek(mCurrentTime*1000);
            mAutoSeek.stop();
            mUserSeeks=false;
        }
    }

    @Override
    public void onStart(MediaView mv) {
        mCurrentTime=mv.getTime()/1000;
        mEndTime=mv.getDuration()/1000;

        mBar.setMax(mEndTime);
        mBar.setProgress(mCurrentTime);
        mMediaView.setOnSeekCompleteListener(this);

        mAutoSeek.start();
        mButton.setImageResource(android.R.drawable.ic_media_pause);
    }

    private class AutoSeek implements Runnable
    {
        private Handler mHandler;
        AutoSeek(Handler h)
        {
            mHandler=h;
        }

        @Override
        public void run() {
            onUpdateSeek();
            mHandler.postDelayed(this, 500);
        }

        public void start()
        {
            mHandler.postDelayed(this, 500);
        }


        public void stop()
        {
            mHandler.removeCallbacks(this);
        }
    }

    protected void onResume()
    {
        super.onResume();
    }

    protected  void onPause()
    {
        mMediaView.stop();
        mAutoSeek.stop();
        super.onPause();

    }

    public void finish()
    {

        mMediaView.stop();
        mAutoSeek.stop();
        mMediaView.clearPlayer();
        super.finish();
    }

    public void onDestroy()
    {
        mMediaView.stop();
        mAutoSeek.stop();
        mMediaView.clearPlayer();
        super.onDestroy();
    }

}
